# 前后端配置和通信

## 1. 与后台通信的功能列表
ueditor和后台通信的功能较多，这里列一下编辑器和后台通信的功能：

1. 上传图片
2. 拖放图片上传、粘贴板图片上传
3. word文档图片转存
4. 截图工具上传
5. 上传涂鸦
6. 上传视频
7. 上传附件
8. 在线图片管理
9. 粘贴转存远程图片

## 2. 参数配置

### 2.1 存在问题
* 修改前后端都需要的配置项，需要注意前后端 **两个地方** 的配置项，而且需要 **对应** 起来。
* 有些常用的配置项直接写在前端代码或者后端代码里面，需要开发者侵入代码修改。

### 2.2 修改方案
* 前后端的配置统一写在后端，编辑器实例化时，异步读取后端配置信息，覆盖到前端的配置里。
* 把散落在代码里的常用配置项，提出来放到后端配置里。

### 2.3 各处配置优先级

后端获取的配置项 > 实例化传入的配置项 > ueditor.config.js文件的配置项。


### 2.4 修改收益
	对前后端通信的配置，开发者只需要关注后端一个文件。


## 3. 统一请求格式

### 3.1 存在问题
* 不同请求发送给不同的文件，请求的数据格式不规范
* 多个功能的返回值不一致，不便于管理代码和二次开发

现有各通信的返回值:
* 上传图片/上传附件/上传视频/word文档图片转存/拖放/粘贴板上传/截图工具上传
```
{'url':'upload/demo.jpg','title':'demo.jpg','original':'demo.jpg','state':'SUCCESS'}
```
* 涂鸦上传(添加背景)
```
<script>parent.ue_callback('upload/demo.jpg','SUCCESS')</script>
```
* 涂鸦上传(上传涂鸦完成后的base64图片)
```
{'url':'upload/demo.jpg',state:'SUCCESS'}
```
* 在线图片管理
```
upload/1.jpgue_separate_ueupload/2.jpg
```
* 远程图片
```
{'url':'upload/1.jpgue_separate_ueupload/2.jpg','tip':'远程图片抓取成功！','srcUrl':'http://img.baidu.com/1.jpgue_separate_uehttp://img.baidu.com/2.jpg'}
```


### 3.2 修改方案
* 通过唯一的后台文件```controller.php```处理前端的请求
* ```controller.php```通过GET上的action参数，判断是什么类型的请求
* 省去不必要的请求，去除涂鸦添加背景的请求，用前端FileReader读取本地图片代替
* 请求返回数据的格式，常规返回json字符串，数据包含state属性（成功时返回'SUCCESS'，错误时返回错误信息）。
* 请求支持jsonp请求格式，当请求有通过GET方式传callback的参数时，返回json数据前后加上括号，再在前面加上callback的值，格式类似这样：```cb({"key": "value"})```

### 3.3 格式规范
以下是各类型的请求说明
#### 3.3.1 config
请求参数:
```
GET {"action": "config"}
POST "upfile": File Data
```
返回格式:
```javascript
// 需要支持callback参数,返回jsonp格式
{
	"imageUrl": "http://localhost/ueditor/php/server.php?action=uploadimage",
	"imagePath": "/ueditor/php/",
	"imageFieldName": "upfile",
	"imageMaxSize": 2048,
	"imageAllowFiles": [".png", ".jpg", ".jpeg", ".gif", ".bmp"]
}
```

#### 3.3.2 uploadimage
请求参数:
```
GET {"action": "uploadimage"}
POST "upfile": File Data
```
返回格式:
```javascript
{
	"state": "SUCCESS",
    "url": "upload/demo.jpg",
	"title": "demo.jpg",
	"original": "demo.jpg"
}
```

#### 3.3.3 uploadscrawl
请求参数:
```
GET {"action": "uploadscrawl"}
POST "upfile": Base64 Data
```
返回格式:
```javascript
{
	"state": "SUCCESS",
    "url": "upload/demo.jpg",
	"title": "demo.jpg",
	"original": "demo.jpg"
}
```
#### 2.3.4 uploadvideo
请求参数:
```
GET {"action": "uploadvideo"}
POST "upfile": File Data
```
返回格式:
```javascript
{
	"state": "SUCCESS",
    "url": "upload/demo.mp4",
	"title": "demo.mp4",
	"original": "demo.mp4"
}
```

#### 3.3.5 uploadfile
请求参数:
```
GET {"action": "uploadfile"}
POST "upfile": File Data
```
返回格式:
```javascript
{
	"state": "SUCCESS",
    "url": "upload/demo.zip",
	"title": "demo.zip",
	"original": "demo.zip"
}
```

#### 3.3.6 listimage
请求参数:
```
GET {"action": "listimage", "start": 0, "size": 20}
```
返回格式:
```javascript
// 需要支持callback参数,返回jsonp格式
{
    "state": "SUCCESS",
    "list": [
        {"url": "upload/1.jpg"},
        {"url": "upload/2.jpg"},
    ],
    "start": 20,
    "total": 100
}
```

#### 3.3.7 catchimage
请求参数:
```
GET {
    "action": "catchimage",
     "source": [
     	"http://a.com/1.jpg",
        "http://a.com/2.jpg"
    ]
}
```
返回格式:
```javascript
// 需要支持callback参数,返回jsonp格式
// list项的state属性和最外面的state格式一致
{
    "state": "SUCCESS",
    "list": [
        {
        	"url": "upload/1.jpg",
        	"source": "http://b.com/2.jpg",
        	"state": "SUCCESS"
        }, {
        	"url": "upload/2.jpg",
        	"source": "http://b.com/2.jpg",
        	"state": "SUCCESS"
        },
    ]
}
```

## 4. 自定义的请求参数
编辑器提供**serverparam**命令，可动态设置自定义参数表。在向后台发出请求时，会把参数表以GET方式加到请求里。

### 4.1 设置自定义参数表
通过serverparam命令设置自定义参数表，**有四种调用方式**，看下面的例子（下文的ue指编辑器实例）：
```javascript
/* 1.传入函数,命令里执行该函数得到参数表,添加到已有参数表里 */
ue.execCommand('serverparam', function(editor){
	return {'key': 'value'};
}};
```
```javascript
/* 2.传入参数表,添加到已有参数表里 */
ue.execCommand('serverparam', {'key': 'value'});
```
```javascript
/* 3.按照键值添加参数 */
ue.execCommand('serverparam', 'key', 'value');
```
```javascript
/* 4.清除参数表 */
ue.execCommand('serverparam'};
```

### 4.2 查询自定义参数表
前端发出请求时，会通过queryCommandValue方法，查询当前自定义参数表，把参数表以GET方式加到请求里：
```javascript
ue.queryCommandValue('serverparam'); //返回参数值键值对的对象
```

### 4.3 使用例子
```javascript
var ue = UE.getEditor('container');
ue.execCommand('serverparam', {
	'key1': 'value1',
	'key2': 'value2',
});
```
提交请求的时候会把key1和key2一起以GET的方式发送到后台。后台再通过$_GET["key1"]和$_GET["key2"]获取key1和key2的值。


## 5. 格式化上传文件名

之前已完成了文件名自定义格式化的功能，由于安全性问题和乱码的问题，暂时没上线功能。现在配置项放到后台比较安全，并且上传工具试用webuploader，编码可控，各版本的语言编码问题测试通过，就可以正式上线这个功能。

附上之前的文档：[链接](http://fex.baidu.com/ueditor/#use-format_upload_filename)


## 6. 旧版本原则

原有ueditor.config.js可以保留使用
