# 增加一个简单按钮
从最简单的功能开始UEditor的二次开发之旅：在工具栏上增加一个简单按钮，点击按钮的时候出现一个alert提示。

## 开发步骤

### 第一步：修改ueditor.config.js文件
在ueditor.config.js文件中，找到toolbars参数，增加一个“showmsg”字符串，对应着添加一个labelMap，用于鼠标移上按钮时的提示。

```javascript
toolbars:[
	[..., 'searchreplace','help','showmsg']
],
labelMap:{
	'anchor':'', 'undo':'','showmsg':'简单按钮'
}
```


### 第二步：修改_src/ui/editorui.js文件
在_src/ui/editorui.js文件中，找到的btnCmds数组变量，在其中同样增加一个"showmsg"字符串。
```javascript
var btnCmds = [..., 'mergecells', 'deletetable', 'showmsg'];
```

这时候，清空缓存刷新下页面！工具栏的对应位置是否出现了一个自己定义的按钮呢？如下图所示：

![新增按钮](images/newbtn.png)


### 第三步：修改themes/default/ueditor.css文件

由于此时未设置对应按钮的图片样式，所以会显示默认的“B”字符。要想让其显示成自己需要的图标样式，接着按照下面的步骤动手吧。

找到themes/default/ueditor.css文件，增加一条样式定义。此处的样式定义了showmsg图标在UEditor默认的精灵Icon图片（themes/default/images/icons.png）中的位置偏移。如需更改成另外图标，只需添加图标到该图片文件中，然后设置偏移值即可。

```css
.edui-for-showmsg .edui-icon {
    background-position: -640px -40px;
}
```

![新增按钮并设置背景](images/newbtn-setstyle.png)


### 第四步：添加_src/plugins/showmsg.js插件文件

到此为止，在UI层面已经完成了一个工具栏图标的显示和各种状态变化的逻辑，但是我们发现点击按钮之后毫无反应。那是必然的，我们还必须为该按钮绑定属于它自己的事件处理方法。

实质上，此时一个默认的事件处理方法已经被UEditor绑定到按钮上了，只不过由于我们还没有定义该方法的具体内容，所以点击之后无任何变化。

下面我们就来定义该方法的具体内容：
找到plugins目录，在该目录中添加一个showmsg.js文件，然后在该文件中输入如下代码：

```javascript
UE.commands['showmsg'] = {
    execCommand : function(){
        alert("Hello,UE developer!");
    }
};
```

然后将该文件引入ueditor：打开_examples/editor_api.js文件，在paths数组中的适当位置（一般是指默认提供的插件部分地址的最后）增加一行plugins/showmsg.js的引用：

```javascript
'plugins/webapp.js',
'plugins/showmsg.js',
'ui/ui.js',
```

再次刷新页面点击一下按钮吧！

![新增按钮并设置动作](images/newbtn-setaction.png)

## 打包文件

注意：正式使用ueditor时，需要把编辑器源码打包成ueditor.all.js文件，具体打包方法请看打包教程：

[使用grunt打包源代码](使用grunt打包源代码.md "拖拽插入和粘贴图片")

